@@ -84,6 +84,12 @@ class DirectoryCleanupMixin(object):
     def create_directory(cls, dir_name):
         return tempfile.mkdtemp(dir=cls.temp_dir, prefix=dir_name)
 
+    @staticmethod
+    def mkdirs(*dirs):
+        for d in dirs:
+            os.makedirs(d)
+        return dirs[:]
+
     def assert_path_does_not_exist(self, *trailing_segments):
         full_path = os.path.join(*trailing_segments)
         if os.path.exists(full_path):
@@ -181,3 +187,119 @@ class EggDirectoryCleanupTests(DirectoryCleanupMixin, unittest.TestCase):
             os.rmdir(installed_egg)
             raise
         os.rmdir(installed_egg)
+
+
+class VirtualEnvironmentCleanupTests(DirectoryCleanupMixin, unittest.TestCase):
+
+    def setUp(self):
+        super(VirtualEnvironmentCleanupTests, self).setUp()
+        self._saved_venv = os.environ.get('VIRTUAL_ENV', None)
+
+    def tearDown(self):
+        super(VirtualEnvironmentCleanupTests, self).tearDown()
+        if self._saved_venv is not None:
+            os.environ['VIRTUAL_ENV'] = self._saved_venv
+        else:
+            os.environ.pop('VIRTUAL_ENV', None)
+
+    def test_that_virtualenv_is_removed_when_dir_is_specified(self):
+        venv_dir = self.create_directory('venv')
+        run_setup(
+            'clean', '--environment', '--virtualenv-dir={0}'.format(venv_dir))
+        self.assert_path_does_not_exist(venv_dir)
+
+    def test_that_virtualenv_is_removed_based_on_envvar(self):
+        venv_dir = self.create_directory('venv')
+        os.environ['VIRTUAL_ENV'] = venv_dir
+        run_setup('clean', '--environment')
+        self.assert_path_does_not_exist(venv_dir)
+
+    def test_that_janitor_does_not_fail_when_envdir_is_missing(self):
+        venv_dir = self.create_directory('venv')
+        os.environ['VIRTUAL_ENV'] = venv_dir
+        os.rmdir(venv_dir)
+        run_setup('clean', '--environment')
+
+    def test_that_janitor_does_not_fail_when_no_dir_specified(self):
+        os.environ.pop('VIRTUAL_ENV', None)
+        run_setup('clean', '--environment')
+
+
+class PycacheCleanupTests(DirectoryCleanupMixin, unittest.TestCase):
+
+    def setUp(self):
+        super(PycacheCleanupTests, self).setUp()
+        self.test_root = self.create_directory('test-root')
+        starting_dir = os.curdir
+        self.addCleanup(os.chdir, starting_dir)
+        os.chdir(self.test_root)
+
+    def test_that_pycache_directories_are_removed(self):
+        all_dirs = self.mkdirs(
+            os.path.join(self.test_root, '__pycache__'),
+            os.path.join(self.test_root, 'a', '__pycache__'),
+            os.path.join(self.test_root, 'a', 'b', '__pycache__'),
+            os.path.join(self.test_root, 'b', '__pycache__'),
+        )
+
+        run_setup('clean', '--pycache')
+        for cache_dir in all_dirs:
+            self.assert_path_does_not_exist(cache_dir)
+
+    def test_that_janitor_does_not_fail_when_cache_parent_is_removed(self):
+        all_dirs = self.mkdirs(
+            os.path.join(self.test_root, 'dist'),
+            os.path.join(self.test_root, 'dist', '__pycache__', '__pycache__'),
+            os.path.join(self.test_root, 'dist', 'foo', '__pycache__'),
+        )
+
+        run_setup('clean', '--dist', '--pycache')
+        for cache_dir in all_dirs:
+            self.assert_path_does_not_exist(cache_dir)
+
+
+class RemoveAllTests(DirectoryCleanupMixin, unittest.TestCase):
+
+    @classmethod
+    def setUpClass(cls):
+        super(RemoveAllTests, cls).setUpClass()
+        test_root = cls.create_directory('test-root')
+        starting_dir = os.curdir
+        saved_env_dir = os.environ.get('VIRTUAL_ENV', None)
+
+        def restore():
+            os.chdir(starting_dir)
+            os.environ.pop('VIRTUAL_ENV', None)
+            if saved_env_dir is not None:
+                os.environ['VIRTUAL_ENV'] = saved_env_dir
+
+        os.chdir(test_root)
+        try:
+            all_dirs = cls.mkdirs(
+                '__pycache__',
+                'dist',
+                'foo.egg-info',
+                'env',
+            )
+            cls.pycache_dir = all_dirs[0]
+            cls.dist_dir = all_dirs[1]
+            cls.egg_dir = all_dirs[2]
+            cls.env_dir = all_dirs[3]
+            os.environ['VIRTUAL_ENV'] = cls.env_dir
+            run_setup('clean', '--all')
+            restore()
+        except:
+            restore()
+            raise
+
+    def test_that_pycache_is_removed(self):
+        self.assert_path_does_not_exist(self.pycache_dir)
+
+    def test_that_distdir_is_removed(self):
+        self.assert_path_does_not_exist(self.dist_dir)
+
+    def test_that_eggdir_is_removed(self):
+        self.assert_path_does_not_exist(self.egg_dir)
+
+    def test_that_envdir_is_removed(self):
+        self.assert_path_does_not_exist(self.env_dir)
